/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.asoftware.letsmeet.beans;

import cz.asoftware.letsmeet.entity.Assignment;
import cz.asoftware.letsmeet.entity.Event;
import cz.asoftware.letsmeet.entity.EventOption;
import cz.asoftware.letsmeet.entity.Participant;
import cz.asoftware.letsmeet.model.ParticipantSelections;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

/**
 *
 * @author aubrech
 */
@Stateless
public class MeetingBeanBean implements MeetingBeanLocal {

    @PersistenceContext
    private EntityManager em;

    @Override
    public List<Event> listAllEvents() {
        List<Event> resultList = em.createNamedQuery("Event.findAll").getResultList();
        // touch all options to get loaded
        for (Event e : resultList) {
            e.getEventOptionCollection().size();
        }
        return resultList;
    }

    @Override
    public List<Participant> listAllParticipants() {
        return em.createNamedQuery("Participant.findAll").getResultList();
    }

    public List<EventOption> listOptionsPerEvent(Event e) {
        List<EventOption> options = new ArrayList<EventOption>(e.getEventOptionCollection());
        Collections.sort(options, new Comparator() {
            public int compare(Object o1, Object o2) {
                //System.err.println("Comparing "+o1+"("+((EventOption)o2).getOptionOrder()+") and "+o2+"("+((EventOption)o1).getOptionOrder()+")");
                return ((EventOption)o1).getOptionOrder() - ((EventOption)o2).getOptionOrder();
            }
        });
        return options;
    }

    public Collection<ParticipantSelections> listParticipantSelections(Event asgEvent) {
        // TODO: tady pridat omezeni na aktualni event a participanta!
        List<Assignment> assgs = em.createNamedQuery("Assignment.listSelections").
                setParameter("asgEventId", asgEvent).
                getResultList();

//        // sort the assignments
//        Collections.sort(assgs, new Comparator() {
//            public int compare(Object o1, Object o2) {
//                return ((Assignment)o1).getAsgOption().getOptionOrder() - ((Assignment)o2).getAsgOption().getOptionOrder();
//            }
//        });

        HashMap<Integer, ParticipantSelections> selections = new HashMap<Integer, ParticipantSelections>();
        for (Assignment as : assgs) {
            ParticipantSelections selection = null;
            Integer participantId = as.getAsgParticipant().getParticipantId();
            if (selections.containsKey(participantId)) {
                selection = selections.get(participantId);
            } else {
                selection = new ParticipantSelections();
                selection.setParticipant(as.getAsgParticipant());
                selections.put(participantId, selection);
            }
            selection.addAssignment(as);
        }
        return selections.values();
    }

    public void saveSelections(ParticipantSelections selectedParticipantSelections) {
        Map<String, String> comments = selectedParticipantSelections.getComments();
        for(String selKey : comments.keySet()) {
            String selVal = comments.get(selKey);
            Assignment as = selectedParticipantSelections.getAssignments().get(selKey);
            as.setAsgComment(selVal);
            as = em.merge(as);
            em.persist(as);
        }
    }

    public void persist(Object object) {
        em.persist(object);
    }
}
